@@ -24,9 +24,15 @@ from collections import Counter
 from io import open
 from multiprocessing import Pool, cpu_count
 
-import torchvision
-import torchvision.transforms as transforms
-from PIL import Image
+try:
+    import torchvision
+    import torchvision.transforms as transforms
+
+    torchvision_available = True
+    from PIL import Image
+except ImportError:
+    torchvision_available = False
+
 from scipy.stats import pearsonr, spearmanr
 from sklearn.metrics import f1_score, matthews_corrcoef
 from tqdm.auto import tqdm
@@ -315,6 +321,7 @@ def convert_examples_to_features(
     sliding_window=False,
     flatten=False,
     stride=None,
+    args=None,
 ):
     """ Loads a data file into a list of `InputBatch`s
         `cls_token_at_end` define the location of the CLS token:
@@ -347,7 +354,11 @@ def convert_examples_to_features(
             with Pool(process_count) as p:
                 features = list(
                     tqdm(
-                        p.imap(convert_example_to_feature_sliding_window, examples, chunksize=500,),
+                        p.imap(
+                            convert_example_to_feature_sliding_window,
+                            examples,
+                            chunksize=args["multiprocessing_chunksize"],
+                        ),
                         total=len(examples),
                         disable=silent,
                     )
@@ -358,7 +369,7 @@ def convert_examples_to_features(
             with Pool(process_count) as p:
                 features = list(
                     tqdm(
-                        p.imap(convert_example_to_feature, examples, chunksize=500),
+                        p.imap(convert_example_to_feature, examples, chunksize=args["multiprocessing_chunksize"]),
                         total=len(examples),
                         disable=silent,
                     )
