@@ -21,7 +21,7 @@ from tqdm.auto import tqdm, trange
 import pandas as pd
 import torch
 from simpletransformers.config.global_args import global_args
-from simpletransformers.custom_models.models import ElectraForQuestionAnswering
+from simpletransformers.custom_models.models import ElectraForQuestionAnswering, XLMRobertaForQuestionAnswering
 from simpletransformers.question_answering.question_answering_utils import (
     RawResult,
     RawResultExtended,
@@ -33,6 +33,7 @@ from simpletransformers.question_answering.question_answering_utils import (
     to_list,
     write_predictions,
     write_predictions_extended,
+    squad_convert_examples_to_features,
 )
 from tensorboardX import SummaryWriter
 from torch.utils.data import DataLoader, RandomSampler, SequentialSampler, TensorDataset
@@ -40,6 +41,9 @@ from torch.utils.data.distributed import DistributedSampler
 from transformers import (
     WEIGHTS_NAME,
     AdamW,
+    AutoConfig,
+    AutoTokenizer,
+    AutoModelForQuestionAnswering,
     AlbertConfig,
     AlbertForQuestionAnswering,
     AlbertTokenizer,
@@ -57,6 +61,8 @@ from transformers import (
     XLMConfig,
     XLMForQuestionAnswering,
     XLMTokenizer,
+    XLMRobertaConfig,
+    XLMRobertaTokenizer,
     XLNetConfig,
     XLNetForQuestionAnswering,
     XLNetTokenizer,
@@ -90,13 +96,15 @@ class QuestionAnsweringModel:
         """  # noqa: ignore flake8"
 
         MODEL_CLASSES = {
-            "bert": (BertConfig, BertForQuestionAnswering, BertTokenizer),
-            "xlnet": (XLNetConfig, XLNetForQuestionAnswering, XLNetTokenizer),
-            "xlm": (XLMConfig, XLMForQuestionAnswering, XLMTokenizer),
-            "distilbert": (DistilBertConfig, DistilBertForQuestionAnswering, DistilBertTokenizer,),
             "albert": (AlbertConfig, AlbertForQuestionAnswering, AlbertTokenizer),
-            "roberta": (RobertaConfig, RobertaForQuestionAnswering, RobertaTokenizer),
+            "auto": (AutoConfig, AutoTokenizer, AutoModelForQuestionAnswering),
+            "bert": (BertConfig, BertForQuestionAnswering, BertTokenizer),
+            "distilbert": (DistilBertConfig, DistilBertForQuestionAnswering, DistilBertTokenizer),
             "electra": (ElectraConfig, ElectraForQuestionAnswering, ElectraTokenizer),
+            "roberta": (RobertaConfig, RobertaForQuestionAnswering, RobertaTokenizer),
+            "xlm": (XLMConfig, XLMForQuestionAnswering, XLMTokenizer),
+            "xlmroberta": (XLMRobertaConfig, XLMRobertaForQuestionAnswering, XLMRobertaTokenizer),
+            "xlnet": (XLNetConfig, XLNetForQuestionAnswering, XLNetTokenizer),
         }
 
         if args and "manual_seed" in args:
@@ -112,11 +120,16 @@ class QuestionAnsweringModel:
             "n_best_size": 20,
             "max_answer_length": 100,
             "null_score_diff_threshold": 0.0,
-            "wandb_project": False,
-            "wandb_kwargs": {},
         }
 
         self.args.update(global_args)
+
+        saved_model_args = self._load_model_args(model_name)
+        if saved_model_args:
+            self.args.update(saved_model_args)
+
+        if args:
+            self.args.update(args)
         self.args.update({"early_stopping_metric": "correct", "early_stopping_metric_minimize": False})
 
         if not use_cuda:
@@ -186,45 +199,44 @@ class QuestionAnsweringModel:
             logger.info(f" Features loaded from cache at {cached_features_file}")
         else:
             logger.info(f" Converting to features started.")
-            features = convert_examples_to_features(
+
+            features, dataset = squad_convert_examples_to_features(
                 examples=examples,
                 tokenizer=tokenizer,
                 max_seq_length=args["max_seq_length"],
                 doc_stride=args["doc_stride"],
                 max_query_length=args["max_query_length"],
                 is_training=not evaluate,
-                cls_token_segment_id=2 if args["model_type"] in ["xlnet"] else 0,
-                pad_token_segment_id=3 if args["model_type"] in ["xlnet"] else 0,
-                cls_token_at_end=True if args["model_type"] in ["xlnet"] else False,
-                sequence_a_is_doc=True if args["model_type"] in ["xlnet"] else False,
-                silent=args["silent"],
+                tqdm_enabled=not args["silent"],
+                threads=args["process_count"],
+                args=args,
             )
 
-            if not no_cache:
-                torch.save(features, cached_features_file)
+            # if not no_cache:
+            #     torch.save(features, cached_features_file)
 
-        all_input_ids = torch.tensor([f.input_ids for f in features], dtype=torch.long)
-        all_input_mask = torch.tensor([f.input_mask for f in features], dtype=torch.long)
-        all_segment_ids = torch.tensor([f.segment_ids for f in features], dtype=torch.long)
-        all_cls_index = torch.tensor([f.cls_index for f in features], dtype=torch.long)
-        all_p_mask = torch.tensor([f.p_mask for f in features], dtype=torch.float)
-        all_example_index = torch.arange(all_input_ids.size(0), dtype=torch.long)
-        if evaluate:
-            dataset = TensorDataset(
-                all_input_ids, all_input_mask, all_segment_ids, all_example_index, all_cls_index, all_p_mask,
-            )
-        else:
-            all_start_positions = torch.tensor([f.start_position for f in features], dtype=torch.long)
-            all_end_positions = torch.tensor([f.end_position for f in features], dtype=torch.long)
-            dataset = TensorDataset(
-                all_input_ids,
-                all_input_mask,
-                all_segment_ids,
-                all_start_positions,
-                all_end_positions,
-                all_cls_index,
-                all_p_mask,
-            )
+        # all_input_ids = torch.tensor([f.input_ids for f in features], dtype=torch.long)
+        # all_input_mask = torch.tensor([f.input_mask for f in features], dtype=torch.long)
+        # all_segment_ids = torch.tensor([f.segment_ids for f in features], dtype=torch.long)
+        # all_cls_index = torch.tensor([f.cls_index for f in features], dtype=torch.long)
+        # all_p_mask = torch.tensor([f.p_mask for f in features], dtype=torch.float)
+        # all_example_index = torch.arange(all_input_ids.size(0), dtype=torch.long)
+        # if evaluate:
+        #     dataset = TensorDataset(
+        #         all_input_ids, all_input_mask, all_segment_ids, all_example_index, all_cls_index, all_p_mask,
+        #     )
+        # else:
+        #     all_start_positions = torch.tensor([f.start_position for f in features], dtype=torch.long)
+        #     all_end_positions = torch.tensor([f.end_position for f in features], dtype=torch.long)
+        #     dataset = TensorDataset(
+        #         all_input_ids,
+        #         all_input_mask,
+        #         all_segment_ids,
+        #         all_start_positions,
+        #         all_end_positions,
+        #         all_cls_index,
+        #         all_p_mask,
+        #     )
 
         if output_examples:
             return dataset, examples, features
@@ -286,10 +298,7 @@ class QuestionAnsweringModel:
             train_dataset, output_dir, show_running_loss=show_running_loss, eval_data=eval_data, **kwargs
         )
 
-        model_to_save = self.model.module if hasattr(self.model, "module") else self.model
-        model_to_save.save_pretrained(output_dir)
-        self.tokenizer.save_pretrained(output_dir)
-        torch.save(self.args, os.path.join(output_dir, "training_args.bin"))
+        self._save_model(model=self.model)
 
         logger.info(" Training of {} model complete. Saved to {}.".format(self.args["model_type"], output_dir))
 
@@ -687,7 +696,7 @@ class QuestionAnsweringModel:
                     "token_type_ids": batch[2],
                 }
 
-                if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra"]:
+                if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra", "xlmroberta"]:
                     del inputs["token_type_ids"]
 
                 example_indices = batch[3]
@@ -817,7 +826,7 @@ class QuestionAnsweringModel:
                     "token_type_ids": batch[2],
                 }
 
-                if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra"]:
+                if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra", "xlmroberta"]:
                     del inputs["token_type_ids"]
 
                 example_indices = batch[3]
@@ -866,7 +875,10 @@ class QuestionAnsweringModel:
                 examples, features, all_results, n_best_size, args["max_answer_length"], False, False, True, False,
             )
 
-        return answers
+        answer_list = [{"id": answer["id"], "answer": answer["answer"]} for answer in answers]
+        probability_list = [{"id": answer["id"], "probability": answer["probability"]} for answer in answers]
+
+        return answer_list, probability_list
 
     def calculate_results(self, truth, predictions, **kwargs):
         truth_dict = {}
@@ -938,7 +950,7 @@ class QuestionAnsweringModel:
             "end_positions": batch[4],
         }
 
-        if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra"]:
+        if self.args["model_type"] in ["xlm", "roberta", "distilbert", "camembert", "electra", "xlmroberta"]:
             del inputs["token_type_ids"]
 
         if self.args["model_type"] in ["xlnet", "xlm"]:
@@ -960,20 +972,36 @@ class QuestionAnsweringModel:
 
         return training_progress_scores
 
-    def _save_model(self, output_dir, optimizer, scheduler, model=None, results=None):
+    def _save_model(self, output_dir=None, optimizer=None, scheduler=None, model=None, results=None):
+        if not output_dir:
+            output_dir = self.args["output_dir"]
         os.makedirs(output_dir, exist_ok=True)
 
-        if model:
+        if model and not self.args["no_save"]:
             # Take care of distributed/parallel training
             model_to_save = model.module if hasattr(model, "module") else model
             model_to_save.save_pretrained(output_dir)
             self.tokenizer.save_pretrained(output_dir)
             torch.save(self.args, os.path.join(output_dir, "training_args.bin"))
-            torch.save(optimizer.state_dict(), os.path.join(output_dir, "optimizer.pt"))
-            torch.save(scheduler.state_dict(), os.path.join(output_dir, "scheduler.pt"))
+            if optimizer and scheduler and self.args["save_optimizer_and_scheduler"]:
+                torch.save(optimizer.state_dict(), os.path.join(output_dir, "optimizer.pt"))
+                torch.save(scheduler.state_dict(), os.path.join(output_dir, "scheduler.pt"))
+            self._save_model_args(output_dir)
 
         if results:
             output_eval_file = os.path.join(output_dir, "eval_results.txt")
             with open(output_eval_file, "w") as writer:
                 for key in sorted(results.keys()):
                     writer.write("{} = {}\n".format(key, str(results[key])))
+
+    def _save_model_args(self, output_dir):
+        os.makedirs(output_dir, exist_ok=True)
+        with open(os.path.join(output_dir, "model_args.json"), "w") as f:
+            json.dump(self.args, f)
+
+    def _load_model_args(self, input_dir):
+        model_args_file = os.path.join(input_dir, "model_args.json")
+        if os.path.isfile(model_args_file):
+            with open(model_args_file, "r") as f:
+                model_args = json.load(f)
+            return model_args
